-- | This module is part of the old @Data.Text.Prettyprint.Doc@ module hierarchy
-- which is being replaced by a shallower @Prettyprinter@ module hierarchy
-- offering the same API.
--
-- This module will be deprecated and eventually removed.
--
-- Use "Prettyprinter.Render.Terminal.Internal" instead.
module Data.Text.Prettyprint.Doc.Render.Terminal.Internal (
    module Prettyprinter.Render.Terminal.Internal
) where

import Prettyprinter.Render.Terminal.Internal


